## R CODE REFERRED TO IN SUPPLEMENTARY MATERIAL SECTION

###############################################
# Calculate statistics on all possible networks

# create a network on 9 nodes
library(ergm)
library(rgeos)
net <- network(9, directed = FALSE)

# calculate all possible vectors of statistics (this takes a while)
stats <- ergm.allstats(net ~ edges + triangles, zeroobs = FALSE, force = TRUE)

# produce the convex hull of the statistic vectors
ch <- gConvexHull(SpatialPoints(stats$statmat)) 

# plot sufficient statistics and convex hull
plot(stats$statmat, pch = 20, main = "Convex Hull", xlab = "Edges", ylab = "Triangles")
plot(ch, add = TRUE)


############################
# Test for existence of MPLE

# create random 9x9 matrix
set.seed(123)
y <- matrix(rbinom(81, 1, 0.3), 9, 9) # Initial matrix
diag(y) <- 0 # set diagonal elements to 0
y <- as.network.matrix(y, directed = FALSE) # turn into network object

# Use ergm's simulated annealing algorithm to find a network with e=26, tri=25
san.net <- san(y ~ edges + triangles, target.stats = c(26, 25) )

# Verify that the new network has 26 edges and 25 triangles
summary(san.net ~ edges + triangles)

# Does the MPLE exist?  Not necessarily; the following code generates a warning:
model <- ergm(san.net ~ edges + triangles, estimate = "MPLE") 

# Plot the network without an MPLE
nine <- seq(0, 2 * pi, len = 10)[-1]
coord <- cbind(cos(nine), sin(nine))
plot(san.net, coord = coord, main = "Network without MPLE")

# Now construct an example where edges=26, triangles=25 and the MPLE exists
# This is less common using a random seed, so define it explicitly
y2 <- 1 - diag(9)
y2[1,5] <- y2[1,7] <- y2[2,4] <- y2[2,8] <- y2[3,5] <- y2[3,7] <- 0
y2[4,6] <- y2[4,9] <- y2[5,7] <- y2[6,9] <- 0
y2 <- y2 * t(y2) # symmetrize
y2 <- as.network.matrix(y2, directed = FALSE) # turn into network object

# Verify that the network has 26 edges and 25 triangles and MPLE exists
summary(y2 ~ edges + triangles)
coef(model <- ergm(y2 ~ edges + triangles, estimate = "MPLE"))

# Plot the network with an MPLE
plot(y2, coord = coord, main = "Network with MPLE")

#######################################################
# Compare confidence interval coverage for four methods

# Set up network with N=50
library(ergm)
init.net <- network(50, directed = FALSE, density = 0.1)
truth <- c(-0.25, -0.2, 0.5) # True rho, sigma, and tau parameter values

# Set up objects to record results
repetitions <- 500
blankmatrix <- matrix(0, nrow = repetitions, ncol = 3,
                      dimnames = list(NULL, c("Edges", "2stars", "Triangles")))
results <- list(mcmle = blankmatrix, fisher = blankmatrix, 
                godambe = blankmatrix, bootstrap = blankmatrix)

# Repeat the simulation 500 times
for(r in 1:repetitions) {

# simulate a network from the ERGM defined by the true parameter values
sim.net <- simulate(init.net~ edges+kstar(2)+ triangles, nsim=1, coef=truth)

# get estimates
mcmle <- ergm(sim.net ~ edges+kstar(2)+ triangles)
fisher <- ergm(sim.net ~ edges+kstar(2)+ triangles, estimate = "MPLE",
      control = control.ergm(MPLE.covariance.method = "invHess"))
godambe <- ergm(sim.net ~ edges+kstar(2)+ triangles, estimate = "MPLE",
      control = control.ergm(MPLE.covariance.method = "Godambe"))
bootstrap <- ergm(sim.net ~ edges+kstar(2)+ triangles, estimate="MPLE",
      control = control.ergm(MPLE.covariance.method="bootstrap"))

# Determine whether the true parameters lie in the z-based 95% confidence interval
# ...for the MLE and corresponding Hessian approximated using MCMC
results$mcmle[r, ] <- (truth > coef(mcmle) - 1.96 * sqrt(diag(mcmle$covar))) &
  (truth < coef(mcmle) + 1.96 * sqrt(diag(mcmle$covar)))

# ...for the naive logistic-regression-based output
results$fisher[r, ] <- (truth > coef(fisher) - 1.96 * sqrt(diag(fisher$covar))) &
  (truth < coef(fisher) + 1.96 * sqrt(diag(fisher$covar)))

# ...for the Godambe approximation of Equation (13)
results$godambe[r, ] <- (truth > coef(godambe) - 1.96 * sqrt(diag(godambe$covar))) &
  (truth < coef(godambe) + 1.96 * sqrt(diag(godambe$covar)))

# ...and for the boostrapped covariance estimate
results$bootstrap[r, ] <- (truth > coef(bootstrap) - 1.96 * sqrt(diag(bootstrap$covar))) &
  (truth < coef(bootstrap) + 1.96 * sqrt(diag(bootstrap$covar)))
}

# Report results as in Table 1
t(sapply(results, colMeans))

###########################
# Create Confidence Elipses

# Set up initial network
library(ergm)
library(ellipse)
set.seed(14392)
N <- 50
y <- matrix(rbinom(N^2, 1, 0.005), N, N)
diag(y) <- 0
y <- as.network.matrix(y, directed = FALSE)

# simulate a network from the true distribution:
init.sim <- simulate(y ~ edges + triangles, nsim = 1, coef = c(-log(N) + 4, -0.2))

# calculate MPLE and approximate Godambe matrix:
m50 <- ergm(init.sim ~ edges + triangles, estimate = "MPLE",
     control=control.ergm(MPLE.covariance.method = "Godambe"))

# calculate inverse Hessian:
m50_2 <- ergm(init.sim ~ edges+triangles, estimate = "MPLE", 
     control=control.ergm(MPLE.covariance.method = "invHess"))

# simulate multiple networks
repetitions <- 1000
sim.net <- simulate(m50, nsim = repetitions) 

# calculate MPLEs of simulated networks: 
sim.coef50 <- matrix(0, nrow = repetitions, ncol = 2)
for(i in 1:repetitions){
  sim.mple <- ergm(sim.net[[i]] ~ edges + triangles, estimate = "MPLE")
  sim.coef50[i,] <- sim.mple$coef
}

# Create plot:
plot(sim.coef50[,1], sim.coef50[,2], xlab = "Edges",
     ylab = "Triangles", main = "N=50", cex.axis = 1.5, cex.lab = 1.5, 
     cex.main=2, pch = 20, col = "darkgray" )
points(m50$coef[1], m50$coef[2], pch = 4)
lines(ellipse(m50_2$covar, centre = m50$coef, level = 0.95), lty = 2, lwd = 2)
lines(ellipse(m50$covar, centre = m50$coef, level = 0.95), lwd = 2)
